package com.unis.framework.common.util.io;

import cn.hutool.core.io.FileTypeUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import lombok.SneakyThrows;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.UUID;

/**
 * 文件工具类
 *
 * @author 芋道源码
 */
public class FileUtils {

    /**
     * 创建临时文件
     * 该文件会在 JVM 退出时，进行删除
     *
     * @param data 文件内容
     * @return 文件
     */
    @SneakyThrows
    public static File createTempFile(String data) {
        File file = createTempFile();
        // 写入内容
        FileUtil.writeUtf8String(data, file);
        return file;
    }

    /**
     * 创建临时文件
     * 该文件会在 JVM 退出时，进行删除
     *
     * @param data 文件内容
     * @return 文件
     */
    @SneakyThrows
    public static File createTempFile(byte[] data) {
        File file = createTempFile();
        // 写入内容
        FileUtil.writeBytes(data, file);
        return file;
    }

    /**
     * 创建临时文件，无内容
     * 该文件会在 JVM 退出时，进行删除
     *
     * @return 文件
     */
    @SneakyThrows
    public static File createTempFile() {
        // 创建文件，通过 UUID 保证唯一
        File file = File.createTempFile(IdUtil.simpleUUID(), null);
        // 标记 JVM 退出时，自动删除
        file.deleteOnExit();
        return file;
    }

    /**
     * 生成文件路径
     *
     * @param content      文件内容
     * @param originalName 原始文件名
     * @return path，唯一不可重复
     */
    public static String generatePath(byte[] content, String originalName, String module) {
//        String sha256Hex = DigestUtil.sha256Hex(content);
        String currentTimeMillis = System.currentTimeMillis() + "_" + UUID.randomUUID().toString().substring(24);
        // 情况一：如果存在 name，则优先使用 name 的后缀
        if (StrUtil.isNotBlank(originalName)) {

            String prefix = FileNameUtil.getPrefix(originalName);//取原文件名
            currentTimeMillis = StrUtil.isBlank(prefix) ? currentTimeMillis : prefix + "_" + currentTimeMillis;

            String extName = FileNameUtil.extName(originalName);
            String path = StrUtil.isBlank(extName) ? currentTimeMillis : currentTimeMillis + "." + extName;
            return generateDateBasedPath(path, module);
        }
        // 情况二：基于 content 计算
        String extName = FileTypeUtil.getType(new ByteArrayInputStream(content));
        String path = currentTimeMillis + '.' + extName;
        return generateDateBasedPath(path, module);
    }

    /**
     * 获取基于日期的默认路径
     * @param fileName 文件名 a.txt
     * @param module 模块名，例如 user
     * @return 全路径名
     */
    public static String generateDateBasedPath(String fileName, String module) {
        module = (StrUtil.isBlank(module)) ? "" : module + "/";
        return module + DateTimeFormatter.ofPattern("yyyy/MM/dd").format(LocalDate.now()) + "/" + fileName;
    }

    public static String generateDateBasedPath(String fileName) {
        return generateDateBasedPath(fileName,null);
    }
}
